package  rand.asset_forfeiture;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

import au.com.bytecode.opencsv.CSVWriter;

/** \mainpage RAND Asset Forfeiture Profit Calculation Tool
 *
 * Put some content here from the report?
 */
/** This is the main workhorse class that organizes and runs the Asset
 * Forfeiture profit calculation tool.
 */
public class AssetGui extends JFrame implements ActionListener {

    /** This JTA stores the results of the current parameters.*/
    private JTextArea backOffice = null;
    /** This JTA stores the results of all cases run since starting
     * the tool.
     */
    private JTextArea resultsLog = null;

    /** The button to launch the WageWizard */
    private JButton wageWizardB = null;
    /** The button to run the calculations */
    private JButton calculateB = null;
    /** The button to clear all the text fields */
    private JButton resetB = null;
    /** The button to close the tool */
    private JButton doneB = null;
    /** The button to bring up more information about the tool */
    private JButton infoB = null;
    /** The button to display backOffice */
    private JButton resultsB = null;
    /** The button to save the current inputs/output to CSV */
    private JButton saveB = null;
    /** The button to display resultsLog */
    private JButton logB = null;

    /** The DisplayPanel which controls all the textual input/output
     * for the variables
     */
    private DisplayPanel dp = null;

    /** The value of PL3 obtained from the BLS */
    private Double defaultPL3 = null;

    /** An array of all the ProfitCalculators that you want to compare */
    private ProfitCalculator[] pcs = null;

    /** The constructor does all the setup, but does not set itself to
     * be visible.
     *
     * \param n the title of the main window
     */
    public AssetGui(String n) {
	super(n);
	setLayout(new BorderLayout());
	setLocation(0,0);

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent wev) {
		    Window w = wev.getWindow();
		    w.setVisible(false);
		    w.dispose();
		}
	    });

	backOffice = new JTextArea();
	backOffice.setText("");
	backOffice.setRows(15);
	backOffice.setEditable(false);

	resultsLog = new JTextArea();
	resultsLog.setText("");
	resultsLog.setRows(15);
	resultsLog.setEditable(false);

	pcs = new ProfitCalculator[3];
	pcs[0] = new KFixed();
	pcs[1] = new KVariable();
	pcs[2] = new Benchmark();

	dp = new DisplayPanel(pcs);

	JPanel buttonPanelT = new JPanel();
	JPanel buttonPanelB = new JPanel();

	wageWizardB = new JButton("Wage Calculation Wizard");
	wageWizardB.addActionListener(this);
	buttonPanelT.add(wageWizardB);

	calculateB = new JButton("Calculate Profits");
	calculateB.addActionListener(this);
	buttonPanelT.add(calculateB);

	resetB = new JButton("Reset Inputs");
	resetB.addActionListener(this);
	buttonPanelB.add(resetB);

	infoB = new JButton("More Information");
	infoB.addActionListener(this);
	buttonPanelB.add(infoB);

	resultsB = new JButton("Detailed Results");
	resultsB.addActionListener(this);
	buttonPanelB.add(resultsB);

	saveB = new JButton("Export to CSV");
	saveB.addActionListener(this);
	buttonPanelB.add(saveB);

	logB = new JButton("View Results Log");
	logB.addActionListener(this);
	buttonPanelB.add(logB);

	doneB = new JButton("Done");
	doneB.addActionListener(this);
	buttonPanelT.add(doneB);

	add(dp, BorderLayout.NORTH);
	add(buttonPanelT,BorderLayout.CENTER);
	add(buttonPanelB,BorderLayout.SOUTH);
	pack();

	CompanyInfo ci = new CompanyInfo(null,
					 "Company Information",
					 true);
	ci.pack();
	ci.setVisible(true);

	defaultPL3 = ci.getWage();
    }

    /** This is the main function for the entire tool. It creates,
     * packes, and sets visible an instance of AssetGui.
     *
     * \param args the command line arguments are ignored
     */
    public static void main(String[] args) {
	AssetGui ag = new AssetGui("Profit Calculation");
	ag.pack();
	ag.setVisible(true);
    }

    /** This is the event handler for all the buttons.
     *
     * \param evt the ActionEvent that spawned this handler
     */
    public void actionPerformed(ActionEvent evt) {
	JButton target = (JButton) evt.getSource();
	if(target == infoB) {
	    JOptionPane.showMessageDialog(null,"More information to come.");
	} else if (target == doneB) {
	    setVisible(false);
	    dispose();
	    System.exit(1);
	} else if (target == resetB) {
	    dp.clear();
	} else if (target == calculateB) {
	    runCalcs();
	} else if (target == wageWizardB) {
	    WageWizard ww = new WageWizard(this,"Wage Wizard",true);
	    ww.setVisible(true);
	    dp.setValues(ww.getValues(), "Inputs");
	} else if (target == resultsB) {
	    JDialog d = new JDialog(this,"Detailed Results",false);
	    JScrollPane jsp = new JScrollPane();
	    jsp.setViewportView(backOffice);
	    d.add(jsp);
	    d.pack();
	    d.setVisible(true);
	} else if (target == logB) {
	    JDialog d = new JDialog(this,"Session Log",false);
	    JScrollPane jsp = new JScrollPane();
	    jsp.setViewportView(resultsLog);
	    d.add(jsp);
	    d.pack();
	    d.setVisible(true);
	} else if (target == saveB) {
	    JFileChooser fc = new JFileChooser();
	    int returnVal = fc.showSaveDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
		File f = fc.getSelectedFile();
		try {
		    f.createNewFile();
		    CSVWriter w = new CSVWriter(new
						FileWriter(f),',');
		    exportCSV(w);
		} catch(IOException e) {
		    System.err.println(e.getMessage());
		}
	    }
	}
    }

    /** This helper function writes the contents of the DisplayPanel
     * to a CSV file.
     *
     * \param w describes the CSV file to write to
     */
    private void exportCSV(CSVWriter w) {
	ArrayList<String[]> lines = new
	    java.util.ArrayList<String[]>();
	String[] l = new String[dp.columnHeadings.length+1];
	l[0] = "";
	for(int i = 1; i < dp.columnHeadings.length+1; i++)
	    l[i] = dp.columnHeadings[i-1];
	lines.add(l);

	for(variables v: variables.values()) {
	    l = new String[dp.columnHeadings.length+1];
	    l[0] = v.notation();
	    for(int i = 1; i < dp.columnHeadings.length+1; i++) {
		String s = dp.columnHeadings[i-1];
		HashMap<variables,Double> data = dp.getValues(s);
		Double val = data.get(v);
		l[i] = val.toString();
	    }
	    lines.add(l);
	}
	w.writeAll(lines);
	try {w.close();}
	catch(IOException e) {
	    System.err.println(e.getMessage());
	}
    }

    /** This helper method does all the processing for a given profit
     * optimization calculation.
     */
    private void runCalcs() {
	backOffice.setText("");

	CustomWage cw = new CustomWage(null,defaultPL3);
	cw.setVisible(true);
	double newPL3 = cw.getNewPL3();

	for(ProfitCalculator calc: pcs) {
	    calc.setVals(dp.getValues("Inputs"));
	    calc.calculateBase(backOffice);
	    calc.setNewPL3(newPL3);
	    if(calc.validate()) {
		calc.calculateExcursion(backOffice);
		dp.setValues(calc.getOptimal(), calc.getName());
	    }
	}
	resultsLog.append(backOffice.getText());
    }

}
