package rand.asset_forfeiture;

import javax.swing.*;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.text.*;

import au.com.bytecode.opencsv.CSVReader;
import java.io.*;
import java.util.HashMap;

/** This is a JDialog extension that interrogates the user about what
 * kind of company is being analyzed. It includes the MOG of the
 * replacement workers, name of the company, and location of the
 * company.
 */
public class CompanyInfo 
    extends JDialog 
    implements ActionListener, ItemListener {

    /** The name of the csv file holding the MOG information. */
    public static final String MOG_FILE = "MOGS_all.csv";

    /** An array to hold the descriptions of all the MOGs */
    public static final String[] mogDescrips = 
	new String[MOGnames.values().length];

    /** A helper function that simply assigns strings to the elements
     * of mogDescrips
     */
    public static void initStrings() {
    mogDescrips[MOGnames.
		A_Professional_And_Technical_Occupations.ordinal()] = 
	"This MOG includes occupations concerned with the study, application,"
	+" and/or administration of physical, mathematical, scientific,"
	+" engineering, architectural, social, medical, legal statute,"
	+" biological, behavioral, library, and/or religious laws, principles,"
	+" practices or theories. Some occupations are concerned with"
	+" interpreting, informing, expressing, or promoting ideas, products,"
	+" etc. by written, artistic, sound or physical mediums. Certain"
	+" occupations that provide support in all of the above fields are"
	+" included in MOG A. Most MOG A occupations require educational"
	+" preparation.\n\n\n"
	+"MOG A also includes technical occupations. These occupations"
	+" are involved in carrying out technical and technological"
	+" functions in health, engineering, science, and other"
	+" disciplines. May perform research, development, testing and"
	+" related activities. May operate technical equipment and"
	+" systems. Such occupations as Laboratory Technicians, Pilots,"
	+" Hydrographers, and Radar Operators are included.";

    mogDescrips[MOGnames.
		B_Executive_Administrative_And_Managerial_Occupations.
		ordinal()] = 
	"Managers plan, organize, direct, and control the major functions of"
	+" an industrial, commercial or governmental establishment or department"
	+" through subordinates who are at the managerial or supervisory"
	+" level. Managers make decisions and establish objectives for the"
	+" department or establishment; they are generally not directly concerned"
	+" with the fabrication of products or with the provision of"
	+" services. They possess a knowledge of the day-to-day operation of the"
	+" organization, but do not necessarily have the detailed knowledge"
	+" required of a first line supervisor. Most Managers are classified in"
	+" MOG B. Please see Section A of the OCSM for more information on"
	+" Managers and Supervisors.\n\n"
	+"In the case of small establishments or departments, employees"
	+" who plan, organize, direct, or control major functions, may also"
	+" perform functions normally assigned to supervisors, such as"
	+" supervising lower level employees. Consider such employees as"
	+" managers.\n\n"
	+"There are also management related occupations in this MOG who"
	+" implement the establishment functions in support of management at the"
	+" operational level. Examples of these specialized functions are"
	+" analyzing financial records and policies, reviewing organizational"
	+" structures and methods, purchasing goods for internal organizational"
	+" use, enforcing standards and regulations, etc.";

    mogDescrips[MOGnames.C_Sales_Occupations.ordinal()] = 
	"Occupations concerned with the selling of goods and services"
	+" or property, purchasing goods and services for resale, or"
	+" conducting wholesale and retail business. Sales"
	+" representatives or agents and sales workers require knowledge"
	+" of the goods or services sold, along with the ability to"
	+" demonstrate product(s), receive payments, and perform other"
	+" sales related activities. Supervisors who coordinate the"
	+" activities of workers who buy and sell goods and services are"
	+" included. Sales clerks and cashiers who are primarily"
	+" concerned with receiving and disbursing funds, and require no"
	+" special product knowledge, are included in MOG C.";

    mogDescrips[MOGnames.
		D_Administrative_Support_Occupations_Including_Clerical.
		ordinal()] = 
	"Include all of the broad groups of occupations performing"
	+" activities relating to preparing, transcribing, systematizing,"
	+" and preserving written communications and records; collecting"
	+" accounts; gathering and distributing information; operating"
	+" office machines and electronic data processing equipment;"
	+" storing, distributing and accounting for stores of materials;"
	+" operating telephone switchboards, distributing mail and"
	+" delivering messages; and performing other administrative and"
	+" clerical support.";

    mogDescrips[MOGnames.
		E_Precision_Production_Craft_And_Repair_Occupations.
		ordinal()] =
	"This group includes occupations involved in the fabricating,"
	+" processing, inspecting or repairing of material, products or"
	+" structural units. Incumbents must have a thorough and"
	+" comprehensive knowledge of processes involved in their work,"
	+" usually acquired through apprenticeship or intensive"
	+" training. Workers must exercise considerable independent"
	+" judgment and must usually display a high degree of manual"
	+" dexterity.\n\n"
	+"Exclude Helpers from MOG E and include them in the indicated"
	+" occupation in MOG H unless specifically included in the"
	+" definition.\n\n"
	+"Apprentices: Include workers who are learning the craft or"
	+" trade through on-the-job training and a formal apprenticeship"
	+" training program.";

    mogDescrips[MOGnames.
		F_Machine_Operators_Assemblers_And_Inspectors.
		ordinal()] = 
	"Set up and operate machinery, perform repetitive manual or"
	+" machine operations, or tend and control machines, as part of a"
	+" fairly well-defined work routine where some independent"
	+" judgment or skill may be required.\n\n"
	+"Include apprentices in MOG F census occupations unless"
	+" specifically excluded.\n\n"
	+"Exclude Helpers in MOG F census occupations unless the job"
	+" description specifically includes them.";

    mogDescrips[MOGnames.
		G_Transportation_And_Material_Moving_Occupations.
		ordinal()] = 
	"Workers concerned with the activities that are in immediate"
	+" support of the operation and performance of transportation"
	+" vehicles used to transport people or material. Includes"
	+" operation of material moving equipment which is stationary or"
	+" has limited range. Also includes the supervision of these"
	+" workers.";

    mogDescrips[MOGnames.
		H_Handlers_Equipment_Cleaners_Helpers_And_Laborers.
		ordinal()] = 
	"Peform unskilled,"
	+" simple duties, primarily manual, that may be learned within a"
	+" short period of time and that require little or no independent"
	+" judgment. These occupations ordinarily require little or no"
	+" previous experience.\n Duties may require moderate to strenuous"
	+" physical exertion.\n\n"
	+" This MOG includes 11 occupations (H483-H498) from the Farm,"
	+" Fishing, and Forestry MOG. These occupations are similar in"
	+" nature to the occupations in MOG H. They are included because"
	+" they occur in the private nonfarm sector.";

    mogDescrips[MOGnames.
		K_Service_Occupations_Except_Private_Households.
		ordinal()] = 
	"Occupations concerned"
	+" with preparing and serving food and drinks in commercial,"
	+" institutional, or other establishments, providing lodging and"
	+" related services, providing grooming, cosmetic and other personal"
	+" and health care services for children and adults, providing"
	+" protection for people and property, attending to the comfort or"
	+" requests of patrons of amusement and recreation facilities, and"
	+" performing cleaning and maintenance services to interiors of"
	+" buildings. These occupations cater to providing personal and"
	+" protective services to individuals and commercial entities.";

    }

    /** The "Done" button */
    private JButton doneB = null;

    /** The model for the company name, so it persists after the
     * dialog is closed.
     */
    private PlainDocument nameModel = null;

    /** The model for the state chooser, so it persists after the
     * dialog is closed.
     */
    private DefaultComboBoxModel stateModel = null;

    /** The menu for choosing an MSA, based on which state is already
     * chosen.
     */
    private JComboBox msa = null;

    /** The model for the MOG chosen, so it persists after the dialog
     * is closed.
     */
    private DefaultComboBoxModel mogs = null;

    /** The only constructor for CompanyInfo does all the setup except
     * for setVisible.
     *
     * \param owner the JFrame that owns the dialog; can be NULL
     * \param title the title of the window
     * \param modal whether this dialog should be modal
     */
    public CompanyInfo(JFrame owner, String title, boolean modal) {
	super(owner, title, modal);
	setLocation(200,200);
	//	setPreferredSize(new Dimension(650,220));

	initStrings();

	SpringLayout manager = new SpringLayout();
	setLayout(manager);

	JTextField name = null;
	JComboBox state = null;

	JLabel label = null;
	JLabel longestLabel = null;

	label = new JLabel("Company Name: ");
	nameModel = new PlainDocument();
	name = new JTextField(nameModel, "", 30);

	add(label);
	add(name);

	manager.putConstraint(SpringLayout.NORTH,
			      label, 1,
			      SpringLayout.NORTH, name);
	label = new JLabel("Choose the occupation of the illegals: ");
	longestLabel = label;

	mogs = new DefaultComboBoxModel(MOGnames.values());
	JComboBox mogBox = new JComboBox(mogs);

	JButton mogInfoButton = new JButton("More Info on this MOG");
	mogInfoButton.setActionCommand("info");
	mogInfoButton.addActionListener(this);

	add(label);
	add(mogBox);
	add(mogInfoButton);

	manager.putConstraint(SpringLayout.NORTH,
			      label, 1,
			      SpringLayout.NORTH, mogBox);
	manager.putConstraint(SpringLayout.NORTH,
			      mogBox, 5,
			      SpringLayout.SOUTH, name);
	manager.putConstraint(SpringLayout.NORTH,
			      mogInfoButton, 5,
			      SpringLayout.SOUTH, label);

	label = new JLabel("State: ");
	stateModel = new DefaultComboBoxModel(States.values());
	state = new JComboBox(stateModel);
	state.addItemListener(this);

	add(label);
	add(state);

	manager.putConstraint(SpringLayout.NORTH,
			      label, 1,
			      SpringLayout.NORTH, state);
	manager.putConstraint(SpringLayout.NORTH,
			      state, 5,
			      SpringLayout.SOUTH, mogInfoButton);

	label = new JLabel("Metropolitan Statistical Area: ");
	msa = new JComboBox();

	add(label);
	add(msa);

	manager.putConstraint(SpringLayout.NORTH,
			      label, 1,
			      SpringLayout.NORTH, msa);
	manager.putConstraint(SpringLayout.NORTH,
			      msa, 5,
			      SpringLayout.SOUTH, state);

	manager.putConstraint(SpringLayout.WEST,
			      name, 5,
			      SpringLayout.EAST, longestLabel);
	manager.putConstraint(SpringLayout.WEST,
			      mogBox, 5,
			      SpringLayout.EAST, longestLabel);
	manager.putConstraint(SpringLayout.WEST,
			      msa, 5,
			      SpringLayout.EAST, longestLabel);
	manager.putConstraint(SpringLayout.WEST,
			      state, 5,
			      SpringLayout.EAST, longestLabel);

	doneB = new JButton("Done");
	add(doneB);
	doneB.addActionListener(this);
	manager.putConstraint(SpringLayout.NORTH,
			      doneB, 5,
			      SpringLayout.SOUTH, msa);

	loadMOGData(MOG_FILE);

	stateModel.setSelectedItem(States.Alabama);
	refreshMSA(States.Alabama);

	manager.putConstraint(SpringLayout.WEST,
			      getContentPane(), 15,
			      SpringLayout.WEST, label);
	manager.putConstraint(SpringLayout.NORTH,
			      getContentPane(), 15,
			      SpringLayout.NORTH, name);
	manager.putConstraint(SpringLayout.EAST,
			      getContentPane(), 15,
			      SpringLayout.EAST, mogBox);
	manager.putConstraint(SpringLayout.SOUTH,
			      getContentPane(), 15,
			      SpringLayout.SOUTH, doneB);

	Dimension dh = doneB.getPreferredSize();
	Dimension dw = manager.preferredLayoutSize(getContentPane());
	Dimension d = new Dimension(dw.width, dh.height);
	doneB.setPreferredSize(d);

	pack();
    }

    /** Handles two different types of GUI events: pressing the Done
     * button or pressing the info button to get more info about a MOG.
     *
     * \param evt the ActionEvent that spawned this listener
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getSource() == doneB)
	    dispose();
	else if(evt.getActionCommand().contentEquals("info")) {
	    JDialog d = new JDialog(this,"MOG Info",true);
	    d.setLocation(getLocation());
	    d.setPreferredSize(getPreferredSize());
	    String t = mogDescrips
		[( (MOGnames) mogs.getSelectedItem()).ordinal()];
	    JTextArea jta = new JTextArea(t,15,30);
	    jta.setEditable(false);
	    jta.setLineWrap(true);
	    jta.setWrapStyleWord(true);
	    JScrollPane jsp = new JScrollPane(jta);
	    d.add(jsp);
	    d.pack();
	    d.setVisible(true);
	}
	else
	    System.err.println(evt.getActionCommand());
    }

    /** Listens for when the state menu changes, so that the list of
     * MSAs is updated correspondingly.
     *
     * \param evt the ItemEvent that spawned this listener
     */
    public void itemStateChanged(ItemEvent evt) {
	if(ItemEvent.SELECTED == evt.getStateChange()) {
	    States state = (States) evt.getItem();
	    refreshMSA(state);
	}
    }

    /** A helper function to set the msa combo box according to the
     * desired state.
     *
     * \param s only populate the msa menu with MSAs in state s
     */
    private void refreshMSA(States s) {
	String mog = getIndustryCode();
	msa.setModel(msasByState.get(mog).get(s));
    }

    /** Accessor for the name of the company
     *
     * \returns the name of the company, unless the nameModel is
     * invalid, in which case it returns the empty string.
     */
    public String getName() {
	try {
	    return nameModel.getText(0, nameModel.getLength());
	}
	catch(BadLocationException e) {
	    return "";
	}
    }

    /** Accessor for the state
     *
     * \returns an instance of States representing the chosen state
     */
    public States getState() {
	return (States) stateModel.getSelectedItem();
    }

    /** Accessor for MSA
     *
     * \returns a string representing the name of the chosen MSA
     */
    public String getMSA() {
	return (String) msa.getModel().getSelectedItem();
    }

    /** Accessor for the MOG industry code
     *
     * \returns a letter representing the BLS MOG industry code
     */
    public String getIndustryCode() {
	MOGnames n = (MOGnames) mogs.getSelectedItem();
	return n.toString().substring(0,1);
    }

    /** These enums represent the columns in the csv MOG file */
    private enum CH {MOG, //!< the MOG code + name
	    state, //!< the state for this entry
	    MSA, //!< the MSA for this entry (blank means statewide)
	    wage //!< the hourly wage for this entry
	    };

    /** This helper method loads all the MOG data into the internal
     * data structures.
     *
     * \param filename the name of the CSV file holding all the data
     */
    private void loadMOGData(String filename) {
	CSVReader reader = null;
	try {
	    reader = new
		CSVReader(new FileReader(filename));
	} catch(FileNotFoundException e) {
	    System.err.println("loadMOGData error: " +
			       e.getMessage());
	}
	String[] nextLine;

	if(reader != null) {
	    try {
		//get rid of the header line
		reader.readNext();
		while((nextLine = reader.readNext()) != null)
		    loadLine(nextLine);
	    } catch(IOException e) {
		System.err.println("loadMOGData error: " +
				   e.getMessage());
	    }
	}
    }

    /** Mapping from MOG to State to the appropariate MSA ComboBox */
    private HashMap<String,
	HashMap<States, DefaultComboBoxModel> > msasByState =
	new HashMap<String,HashMap<States,DefaultComboBoxModel>>();

    /** Mapping from MOG to MSA to average wage */
    private HashMap<String,
	HashMap<String,Double> > wagesByMSA = 
	new HashMap<String,HashMap<String,Double>>();

    /** Accessor to get the yearly expense for a worker in the
     * selected MOG and MSA.
     *
     * \returns the yearly expense for the selected category,
     * including benefits, social security, and medicare
     */
    public Double getWage() {
	double f = WageWizard.HOURS_PER_YEAR *
	    (1 + WageWizard.TOTAL_BENEFITS + WageWizard.SS_MEDICARE);

	String mog = getIndustryCode();
	Double wage = wagesByMSA.get(mog).
	    get((String) msa.getSelectedItem());
	if(wage == null) {
	    States region = 
		((States) stateModel.getSelectedItem()).getRegion();
	    String fauxMSA = region.toString() + ": Other";
	    wage = wagesByMSA.get(mog).get(fauxMSA);
	}

	return f*wage;
    }

    /** This helper method takes a line of output from a CSVReader or
     * similar and does the grunt work populating msasByState and
     * wagesByMSA.
     *
     * \param line an array of strings, each one representing a column
     * from one row of the MOG CSV file.
     */
    private void loadLine(String[] line) {
	if(line.length < CH.values().length)
	    return;

	String MOG = line[CH.MOG.ordinal()].substring(0,1);

	int i = line[CH.state.ordinal()].indexOf("=");
	String sString =
	    line[CH.state.ordinal()].substring(i+2).trim();
	States s;
	try {
	    s = States.fromString(sString);
	} catch(IllegalArgumentException e) {
	    System.err.println("loadLine error: " + e.getMessage());
	    return;
	}

	i = line[CH.MSA.ordinal()].indexOf("=");
	String MSA;
	if(i >= 0)
	    MSA = line[CH.MSA.ordinal()].substring(i+2).trim();
	else
	    MSA = sString + ": Other";

	String wString = line[CH.wage.ordinal()].trim();
	Double w;
	if(wString.contentEquals("."))
	    w = null;
	else
	    w = new Double(wString);


	if(!msasByState.containsKey(MOG))
	    msasByState.put(MOG, new
			    HashMap<States,DefaultComboBoxModel>());
	if(!wagesByMSA.containsKey(MOG))
	    wagesByMSA.put(MOG, new HashMap<String,Double>());

	wagesByMSA.get(MOG).put(MSA,w);
	if(!msasByState.get(MOG).containsKey(s))
	    msasByState.get(MOG).put(s, new DefaultComboBoxModel());
	msasByState.get(MOG).get(s).addElement(MSA);

    }
}
