package rand.asset_forfeiture;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/** This class is a simple extension of JDialog to interrogate the
 * user for a value for newPL3 (the wage paid to workers hired to
 * replace the illegal immigrants).
 */
public class CustomWage extends JDialog implements ActionListener {

    /** The input box to accept a user-input value of newPL3 */
    private JTextField newWage = null;
    /** The text label asking for input */
    private JLabel wagePrompt = null;
    /** The BLS take on what newPL3 should be */
    private Double defaultPL3 = null;

    /** The only constructor for CustomWage sets up all the
     * components, but does not set the dialog to visible. By default,
     * CustomWage is modal.
     *
     * \param owner the JFrame owning this JDialog; can be NULL
     * \param defW the default value for newPL3, usually the BLS value
     */
    public CustomWage(JFrame owner, Double defW) {
	super(owner, "Modify Wages?", true);
	setLocation(300,300);
	defaultPL3 = defW;

	GridBagLayout layout = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.anchor = GridBagConstraints.NORTH;

	setLayout(layout);

	JButton okB = new JButton("Done");
	okB.setActionCommand("ok");
	okB.addActionListener(this);

	JLabel q1 = new JLabel("By default, replacement workers for"
			       +" the illegal workers will be hired"
			       +" at the BLS FTE yearly expense.");

	wagePrompt = new JLabel("You may modify this yearly "
				+"expense if you wish.");

	newWage = new JTextField(defaultPL3.toString());
	newWage.setHorizontalAlignment(JTextField.CENTER);
	newWage.setMaximumSize(newWage.getPreferredSize());

	add(q1);
	layout.setConstraints(q1,gbc);
	add(wagePrompt);
	layout.setConstraints(wagePrompt,gbc);
	add(newWage);
	layout.setConstraints(newWage, gbc);
	add(okB);
	layout.setConstraints(okB, gbc);

	pack();

    }

    /** The accessor for newPL3.
     *
     * \returns 0 if the text in newWage is not a valid number,
     * otherwise the number represented by newWage
     */
    public Double getNewPL3() {
	Double w = null;
	try {
	    w = new Double(newWage.getText().replace(",",""));
	} catch(NumberFormatException e) {
	    w = 0.0;
	}

	return w;
    }

    /** Catches the pressing of the OK button when the user is
     * finished updating the default wage.
     *
     * \param evt the ActionEvent that this listener
     * catches. Currently, the only possibility is the OK button
     */
    public void actionPerformed(ActionEvent evt) {
	if(evt.getActionCommand().contentEquals("ok")) {
	    dispose();
	}
    }
}