package rand.asset_forfeiture;

import javax.swing.*;
import java.util.*;

/** KFixed calculates the optimum allocation of resources for a firm
 * that can no longer hire illegal immigrants and has the additional
 * constraint of fixed capital costs.
 */
public class KFixed extends GCalc {

    /** The displayed name of this class, used for text output. */
    private static String name = "K Fixed";

    /** The only constructor simply calls super() */
    public KFixed()
    {
	super();
    }

    //documented in ProfitCalculator
    public String getName(){return name;}

    //documented in ProfitCalculator
    public Boolean needsVar(variables v)
    {
	switch(v) {
	case L1:
	    return true;
	case L3:
	    return true;
	case L2:
	    return true;
	case PL1:
	    return true;
	case PL3:
	    return true;
	case PL2:
	    return false;
	case R:
	    return true;
	case pi:
	    return true;
	case rho:
	    return true;
	default:
	    return false;
	}
    }

    //these are all documented in GCalc
    protected Double X3Opt() {
	return Math.max(1, phi3() * delta() / PL3());
    }

    protected String X3OptString() {
	return "phi3 * delta / PL3";
    }

    protected Double KOpt() {
	return K();
    }

    protected String KOptString() {
	return "K";
    }

    protected Double phi3() {
	Double num = Math.pow(PL1(),beta()) *
	    Math.pow(PL4(),gamma()) *
	    Math.pow(PL3(),delta());
	Double denom = Pq() * Math.pow(K(),alpha()) *
	    Math.pow(beta(),beta()) * Math.pow(gamma(),gamma()) *
	    Math.pow(delta(),delta()) * Math.pow(epsilon(),epsilon());
	Double exp = 1/(beta() + gamma() + delta() + epsilon() - 1);

	return Math.pow(num/denom,exp);
    }

    protected String phi3String() {
	return " ( (PL1^b * PL4^g * PL3^d) / (Pq * K^a * b^b * g^g *"
	    + " d^d * e^e) ) ^ (1 / (b + g + d + e -1) )";
    }

}
