package rand.asset_forfeiture;

import javax.swing.*;
import java.util.*;

/** KVariable calculates the optimum allocation of resources if the
 * firm can no longer hire illegal immigrants, but capital and raw
 * material costs are completely flexible.
 */
public class KVariable extends GCalc {

    /** The displayed name of this class, used for text output. */
    private static String name = "K Variable";

    /** The only constructor simply calls super() */
    public KVariable()
    {
	super();
    }

    //documented in ProfitCalculator
    public String getName(){return name;}

    //documented in ProfitCalculator
    public Boolean needsVar(variables v)
    {
	switch(v) {
	case L1:
	    return true;
	case L3:
	    return true;
	case L2:
	    return true;
	case PL1:
	    return true;
	case PL3:
	    return true;
	case PL2:
	    return false;
	case R:
	    return true;
	case pi:
	    return true;
	default:
	    return false;
	}
    }

    // These are all documented in GCalc
    protected Double phi3() {
	Double num = Math.pow(PL1(),beta()) *
	    Math.pow(PL4(),gamma()) * Math.pow(PL3(),delta());
	Double denom = Pq() * Math.pow(alpha(),alpha()) *
	    Math.pow(beta(),beta()) * Math.pow(gamma(),gamma()) *
	    Math.pow(delta(),delta()) * Math.pow(epsilon(),epsilon());
	Double exp = 1/(alpha() + beta() + gamma() + delta() +
			epsilon() - 1);

	return Math.pow(num/denom,exp);
    }

    protected String phi3String() {
	return  " ( (PL1^b * PL4^g * PL3^d) / (Pq * a^a * b^b * g^g *"
	    + " d^d * e^e) ) ^ (1 / (b + g + d + e -1) )";
    }

    protected Double KOpt() {
	return Math.max(1, PL3()*X3Opt()*alpha()/delta());
    }

    protected String KOptString() {
	return "PL3 * X3* * alpha / delta";
    }

    protected Double X3Opt() {
	return Math.max(1, phi3() * delta() / PL3());
    }

    protected String X3OptString() {
	return "phi3 * d / PL3";
    }
}
