package rand.asset_forfeiture;

/** This enumeration describes all the possible MOGs that one can pick
 * to represent the illegal workers' occupations. The first letter of
 * each member is its corresponding BLS code. The text following that
 * letter describes the scope of the MOG.
 */
public enum MOGnames {
    /** Profesional and technical */
    A_Professional_And_Technical_Occupations,
	/** Administrative and Managerial */
	B_Executive_Administrative_And_Managerial_Occupations,
	/** Sales and Retail */
	C_Sales_Occupations,
	/** Administrative support */
	D_Administrative_Support_Occupations_Including_Clerical,
	/** Precision Production, Crafts, and Repair */
	E_Precision_Production_Craft_And_Repair_Occupations,
	/** Machine Operators */
	F_Machine_Operators_Assemblers_And_Inspectors,
	/** Transportation */
	G_Transportation_And_Material_Moving_Occupations,
	/** Manual Labor */
	H_Handlers_Equipment_Cleaners_Helpers_And_Laborers,
	/** Service Occupations */
	K_Service_Occupations_Except_Private_Households;

    /** A custom fromString converter, since the strings in the MOG
     * database have spaces, but we need to use underscores to keep java
     * happy.
     * 
     * \param v a string representation of the varible, with either
     * spaces or underscores between words
     *
     * \returns the corresponding MOGnames instance
     *
     * \throws an IllegalArgumentException if v is not a valid MOG
     */
    public static MOGnames fromString(String v) {

	v = v.replaceAll(",","");

	for(MOGnames s: MOGnames.values()) {
	    String mString = s.toString().substring(2);
	    mString = mString.replaceAll("_"," ");
	    if(v.contentEquals(mString))
		return s;
	}
	throw(new IllegalArgumentException(v + " is not an MOG."));
    }

    /** A custom toString() method that strips out the underscores.
     *
     * \returns a string containing the one-letter code for the MOG
     * and a more descriptive name
     */
    public String toString() {
	switch(this) {
	case A_Professional_And_Technical_Occupations:
	    return "A: Professional and Technical Occupations";
	case B_Executive_Administrative_And_Managerial_Occupations:
	    return
		"B: Executive, Administrative, and Managerial Occupations";
	case C_Sales_Occupations:
	    return "C: Sales Occupations";
	case D_Administrative_Support_Occupations_Including_Clerical:
	    return
		"D: Administrative Support Occupations, Including Clerical";
	case E_Precision_Production_Craft_And_Repair_Occupations:
	    return
		"E: Precision Production, Craft, and Repair Occupations";
	case F_Machine_Operators_Assemblers_And_Inspectors:
	    return 
		"F: Machine Operators Assemblers and Inspectors";
	case G_Transportation_And_Material_Moving_Occupations:
	    return
		"G: Transportation and Material Moving Occupations";
	case H_Handlers_Equipment_Cleaners_Helpers_And_Laborers:
	    return
		"H: Handlers, Equipment Cleaners, Helpers, and Laborers";
	case K_Service_Occupations_Except_Private_Households:
	    return
		"K: Service Occupations, Except Private Households";
	default:
	    return "Not a valid MOG";
	}
    }
}