package rand.asset_forfeiture;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

/** An improvement over the DefaultTableCellRenderer specific to text
 * cells. It colors the output cells Color.GRAY to differentiate them
 * from user-editable cells and it also sets the cell background color
 * when it has focus to alert the user that the cell contents have
 * been selected.
 */
public class MyTextBoxRenderer 
    extends DefaultTableCellRenderer {

    /** Pulling in the default selection color */
    private Color selectionBackground = UIManager.
	getColor("TextField.selectionBackground");

    /** Setting the cell borders to Color.BLACK ensures that they are
     * clearly visible, even when some of the cell backgrounds are
     * Color.GRAY.
     */
    private Border editBorder =
	BorderFactory.createLineBorder(Color.BLACK);

    /** Tracks whether the cell this object renders currently has
     * focus or not.
     */
    private boolean cellHasFocus;
    
    /** Reimplementing DefaultTableCellRenderer.getTableCellRenderer
     * by calling the super version, recording whether the cell has
     * focus, and setting the background color if the cell in question is
     * an output cell.
     * 
     * \param table the JTable this cell belongs to
     * \param value the current value of the cell
     * \param isSelected whether this cell is currently selected
     * \param hasFocus whether this cell currently has input focus
     * \param row the row of this cell in table
     * \param column the column of this cell in table
     * 
     * \returns this
     */
    public Component getTableCellRendererComponent(JTable table,
						   Object value,
						   boolean isSelected,
						   boolean hasFocus,
						   int row,
						   int column) {
	super.getTableCellRendererComponent(table, value, isSelected,
					    hasFocus, row, column);
	
	cellHasFocus = hasFocus;
	
	if(column >= WageWizard.columnHeadings.fte.ordinal())
	    setBackground(Color.GRAY);
	return this;
    }

    /** Reimplementation of super.paintComponent by setting the
     * background color to the selectionBackground if the cell has focus
     * and is non-empty then calling super.paintComponent.
     * 
     * \param g the graphics object used to paint this component
     */    
    protected void paintComponent(Graphics g)
    {
	if (cellHasFocus && !getText().equals(""))
	    {
		setBorder( editBorder );
		g.setColor( selectionBackground );
		g.fillRect(0, 0, getPreferredSize().width, getSize().height);
	    }
	
	super.paintComponent(g);
    }
    
}
