package rand.asset_forfeiture;

import javax.swing.JTable;
import javax.swing.table.*;
import java.awt.Component;

/** We need an enhancement to the DefaultTableModel in order to
 * properly render combo boxes in individual cells as well as prevent
 * the user from editing the output cells.
 */
public class WageTableModel extends DefaultTableModel {

    /** You only get one constructor, and it just calls its super.
     * 
     * \param colNames and Object array that should just hold the
     * names of each column in order you want them displayed. It is
     * assumed that all the output cells are after all the input cells.
     */
    public WageTableModel(Object[] colNames) {
	super(colNames,0);
    }

    /** This over rides the original pack in order to auto-size the
     * column widths properly so all column names are readable.
     * 
     * \param table the JTable that needs to be packed.
     */
    public void pack(JTable table) {
	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	for(int i=0; i < getColumnCount(); i++) {
	    TableColumn col = table.getColumnModel().getColumn(i);
	    TableCellRenderer rend = col.getHeaderRenderer();
	    if(rend == null)
		rend = table.getTableHeader().getDefaultRenderer();
	    Component comp = rend.
		getTableCellRendererComponent(
					      table,
					      col.getHeaderValue(),
					      false,
					      false,
					      0, 0);

	    int width = comp.getPreferredSize().width;
	    rend = table.getCellRenderer(0, i);
	    comp = rend.
		getTableCellRendererComponent(
					      table,
					      table.getValueAt(0, i),
					      false, false, 0, i);
	    width = Math.max(width, comp.getPreferredSize().width);
	    col.setPreferredWidth(width + 4);
	}
    }

    /** Overriding the regular getColumnClass method so that not
     * everything is rendered as a String (I want my checkboxes!).
     * 
     * \param column the integer for which column you want class
     * information about.
     * 
     * \returns the Class object describing what type of data the
     * indicated column holds.
     */
   public Class getColumnClass(int column) {
	return getValueAt(0,column).getClass();
    }

    /** Overriding the default, which is always true, since the user
     * should not be allowed to edit the output cells.
     *
     * \param row the row of the cell in question
     * \param column the column of the cell in question
     * 
     * \returns false if the cell is not editable; true if it is
     */
    public boolean isCellEditable(int row, int column) {
	if(column > WageWizard.columnHeadings.wpy.ordinal())
	    return false;
	else
	    return true;
    }

}
