package rand.asset_forfeiture;

/** These are all the input variables that can be used in the
 * DisplayPanel and ProfitCalculator.
 */
public enum variables{
    L1, //!< number of legal, skilled workers
	L3, //!< number of legal, unskilled workers
	L2, //!< number of illegal, unskilled workers (or replacements)
	PL1, //!< yearly expense per FTE of L1
	PL3, //!< yearly expense per FTE of L3
	PL2, //!< yearly expense per FTE of L2
	R, //!< yearly total revenues
	rho, //!< yearly raw materials costs
	pi, //!< yearly pre-tax profits
	diff; //!< the difference in profit between original and optimization

    /** A custom string conversion method to create much more legible
     * variable names on the input side.
     *
     * \returns a string describing the input value
     */
    public String toString() {
	switch(this) {
	case L1: return "Legal Skilled/Managerial FTE";
	case L2: return "Illegal Unskilled FTE";
	case L3: return "Legal Unskilled FTE";
	case PL1: return "Expense per FTE Legal Skilled Worker";
	case PL2: return "Expense per FTE Illegal Unskilled Worker";
	case PL3: return "Expense per FTE Legal Unskilled Worker";
	case R: return "Revenue";
	case pi: return "Pre-tax Profit";
	case rho: return "Raw Materials and Supplies";
	case diff: return "";
	}
	return "Unknown variables";
    }

    /** A custom string conversion method to create more legible
     * variables names on the output side.
     *
     * \returns a string describing the output value
     */
    public String toAlternateString() {
	switch(this) {
	case L1: return "Optimized Legal Skilled/Managerial FTE";
	case L2: return "Optimized Replacement FTE for Illegals";
	case L3: return "Optimized Legal Unskilled FTE";
	case PL1: return "Expense per FTE Legal Skilled Worker";
	case PL2: return "Expense per FTE Replacement Worker";
	case PL3: return "Expense per FTE Legal Unskilled Worker";
	case R: return "Optimized Revenue";
	case pi: return "Optimized Pre-tax Profit";
	case rho: return "Optimized Raw Materials and Supplies";
	case diff: return "Profit due to Illegals";
	}
	return "uknown variables";
    }

    /** A string conversion method to convert from our internal
     * variable name to the notation used in the derivation.
     *
     * \returns a string with the corresponding notation for this
     * variable
     */
    public String notation() {
	switch(this) {
	case L1: return "X1";
	case L2: return "X2/X4";
	case L3: return "X3";
	case PL1: return "PL1";
	case PL2: return "PL2/PL4";
	case PL3: return "PL3";
	case R: return "R";
	case pi: return "pi";
	case rho: return "rho";
	}
	return "variable " + toString() + " has no notation";
    }

    /** Converts a stirng of the toString() variety into its
     * corresponding variable.
     *
     * \param s the string to be converted
     *
     * \returns the corresponding member of variables; null if s is
     * invalid
     */
    public static variables fromString(String s) {
	for(variables v: variables.values()) {
	    if(v.toString().contentEquals(s))
		return v;
	}
	return null;
    }

    /** Returns the help string for each variable.
     *
     * \returns the appropriate help string
     */
    public String getHelp() {
	switch(this) {
	case L1: return L1Help;
	case L2: return L2Help;
	case L3: return L3Help;
	case PL1: return PL1Help;
	case PL2: return PL2Help;
	case PL3: return PL3Help;
	case R: return RHelp;
	case pi: return piHelp;
	case rho: return rhoHelp;
	}
	return "Variable has no help file.";
    }

    /** The help string for L1 */
    private static String L1Help = "Enter the number of full time"
    +" equivalent skilled, managerial, and administrative workers. Simply"
    +" put, if each employee worked exactly 40 hours per week,"
    +" how many employees (fractional employees allowed) would"
    +" need to be hired?";

    /** The help string for L3 */
    private static String L3Help = "Enter the number of full time"
    +" equivalent blue collar, legal workers. Simply"
    +" put, if each employee worked exactly 40 hours per week,"
    +" how many employees (fractional employees allowed) would"
    +" need to be hired?";

    /** The help string for L2 */
    private static String L2Help = "Enter the number of full time"
    +" equivalent illegal workers. The model assumes that these"
    +" workers are all of a similar class (e.g. unskilled manual"
    +" labor).  Simply"
    +" put, if each employee worked exactly 40 hours per week,"
    +" how many employees (fractional employees allowed) would"
    +" need to be hired?";

    /** The help string for PL1 */
    private static String PL1Help = "Enter the average expense"
    +" for each full time equivalent skilled, managerial, and"
    +" administrative worker. Take the total bill (including"
    +" benefits and employer-paid taxes) for all these workers"
    +" and divide by the FTE entered above.";

    /** The help string for PL3 */
    private static String PL3Help = "Enter the average expense"
    +" for each full time equivalent legal, blue collar"
    +" worker. Take the total bill (including"
    +" benefits and employer-paid taxes) for all these workers"
    +" and divide by the FTE entered above.";

    /** The help string for PL2 */
    private static String PL2Help = "Enter the average expense"
    +" for each full time equivalent illegal"
    +" worker. Take the total bill (including and benefits or"
    +" taxes paid) for all these workers"
    +" and divide by the FTE entered above.";

    /** The help string for R */
    private static String RHelp = "Enter the total gross"
    +" revenues for the firm during the year in question.";

    /** The help string for pi */
    private static String piHelp = "Enter the pre-tax profits"
    +" earned by the firm during the year in question.";

    /** The help string for rho */
    private static String rhoHelp = "Enter the total amount"
    +" spent by the firm on raw materials (i.e. consumables)"
    +" during the year in question. This includes both materials"
    +" incoprorated into products (e.g. lumber for carpentry)"
    +" as well as supplies (e.g. fuel or cleaning solution).";

}
